from xmltodict import parse
from datetime import datetime
from .area import Area

class Weather:
    def __init__(self, response, **settings):
        json = parse(response)["data"]
        self.source = json["@source"]
        self.production_center = json["@productioncenter"]
        self.areas = []
        self.date = datetime(
            int(json["forecast"]["issue"]["year"]),
            int(json["forecast"]["issue"]["month"]),
            int(json["forecast"]["issue"]["day"]),
            int(json["forecast"]["issue"]["hour"]),
            int(json["forecast"]["issue"]["minute"]),
            int(json["forecast"]["issue"]["second"])
        )
        
        for area in json["forecast"]["area"]:
            self.areas.append(Area(area, **settings))
    
    def __repr__(self):
        return f"<Weather date={repr(self.date)} areas=[{len(self.areas)}]>"


from datetime import datetime
from .constants import WEATHER_CODE, WIND_DIRECTION_CODE

class Area:
    def __repr__(self):
        return f"<Area id={self.id} name={self.name} latitude={self.latitude} longitude={self.longitude}>"

    def __init__(self, data, **settings):
        self._settings       = settings
        self.id              = int(data["@id"])
        self.name            = data["name"][0 if settings["english"] else 1]["#text"]
        self.latitude        = float(data["@latitude"])
        self.longitude       = float(data["@longitude"])
        self.type            = data["@type"]
        self.level           = int(data["@level"])
        self.humidity_type   = data["parameter"][0]["timerange"][0]["@type"]
        self.url             = f"https://www.bmkg.go.id/cuaca/prakiraan-cuaca.bmkg?AreaID={self.id}"
        
        self.humidity        = list(map(self._parse_humidity, data["parameter"][0]["timerange"]))
        self.max_humidity    = list(map(self._parse_humidity, data["parameter"][1]["timerange"]))
        self.min_humidity    = list(map(self._parse_humidity, data["parameter"][3]["timerange"]))
    
        self.max_temperature = list(map(self._parse_temperature, data["parameter"][2]["timerange"]))
        self.min_temperature = list(map(self._parse_temperature, data["parameter"][4]["timerange"]))
        self.temperature     = list(map(self._parse_temperature, data["parameter"][5]["timerange"]))
        
        self.wind_speeds     = list(map(lambda x: {
            "datetime": datetime(
                int(x["@datetime"][:4]),
                int(x["@datetime"][4:6]),
                int(x["@datetime"][6:8]),
                int(x["@datetime"][8:10]),
                int(x["@datetime"][10:])
            ),
            "value": float(x["value"][2 if self._settings["metric"] else 1]["#text"]),
            "ms": float(x["value"][3]["#text"]),
            "knots": float(x["value"][0]["#text"])
        }, data["parameter"][8]["timerange"]))
    
        self.wind_direction  = list(map(lambda x: {
            "datetime": datetime(
                int(x["@datetime"][:4]),
                int(x["@datetime"][4:6]),
                int(x["@datetime"][6:8]),
                int(x["@datetime"][8:10]),
                int(x["@datetime"][10:])
            ),
            "value": float(x["value"][0]["#text"]),
            "text": WIND_DIRECTION_CODE[x["value"][1]["#text"]],
            "sexa": float(x["value"][2]["#text"])
        }, data["parameter"][7]["timerange"]))
    
        self.forecast        = list(map(lambda x: {
            "datetime": datetime(
                int(x["@datetime"][:4]),
                int(x["@datetime"][4:6]),
                int(x["@datetime"][6:8]),
                int(x["@datetime"][8:10]),
                int(x["@datetime"][10:])
            ),
            "value": WEATHER_CODE[x["value"]["#text"]][0],
            "icon_url": WEATHER_CODE[x["value"]["#text"]][1].format("am" if int(x["@datetime"][8:10]) < 12 else "pm")
        }, data["parameter"][6]["timerange"]))
    
    def _parse_temperature(self, data):
        return {
            "datetime": datetime(
                int(data["@datetime"][:4]),
                int(data["@datetime"][4:6]),
                int(data["@datetime"][6:8]),
                int(data["@datetime"][8:10]),
                int(data["@datetime"][10:])
            ),
            "value": float(data["value"][0 if self._settings["metric"] else 1]["#text"])
        }
    
    def _parse_humidity(self, data):
        return {
            "datetime": datetime(
                int(data["@datetime"][:4]),
                int(data["@datetime"][4:6]),
                int(data["@datetime"][6:8]),
                int(data["@datetime"][8:10]),
                int(data["@datetime"][10:])
            ),
            "value": int(data["value"]["#text"]),
            "unit": data["value"]["@unit"]
        }
    
    def __int__(self):
        return self.id


DIRECTION = {
    "BaratDaya": "southwest",
    "BaratLaut": "northwest",
    "Tenggara": "southeast",
    "TimurLaut": "northeast",
    "Utara": "north",
    "Selatan": "south",
    "Timur": "east",
    "Barat": "west"
}

TIMEZONE_OFFSETS = {
    "WIB": 7,
    "WITA": 8,
    "WIT": 9
}

WIND_DIRECTION_CODE = {
    "N": "North",
    "NNE": "North-Northeast",
    "NE": "Northeast",
    "ENE": "East-Northeast",
    "E": "East",
    "ESE": "East-Southeast",
    "SE": "Southeast",
    "SSE": "South-Southeast",
    "SSW": "South-Southwest",
    "SW": "Southwest",
    "WSW": "West-Southwest",
    "W": "West",
    "S": "South",
    "WNW": "West-Northwest",
    "NW": "Northwest",
    "NNW": "North-Northwest",
    "VARIABLE": "Fluctuate"
}

WEATHER_CODE = {
    "0":   ("Clear Skies", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah-{}.png"),
    "1":   ("Partly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah%20berawan-{}.png"),
    "2":   ("Partly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah%20berawan-{}.png"),
    "3":   ("Mostly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/berawan-{}.png"),
    "4":   ("Overcast", "https://www.bmkg.go.id/asset/img/icon-cuaca/berawan%20tebal-{}.png"),
    "5":   ("Haze", "https://www.bmkg.go.id/asset/img/icon-cuaca/udara%20kabur-{}.png"),
    "10":  ("Smoke", "https://www.bmkg.go.id/asset/img/icon-cuaca/asap-{}.png"),
    "45":  ("Fog", "https://www.bmkg.go.id/asset/img/icon-cuaca/kabut-{}.png"),
    "60":  ("Light Rain", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20ringan-{}.png"),
    "61":  ("Rain", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20sedang-{}.png"),
    "63":  ("Heavy Rain", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20lebat-{}.png"),
    "80":  ("Isolated Shower", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20lokal-{}.png"),
    "95":  ("Severe Thunderstorm", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20petir-{}.png"),
    "97":  ("Severe Thunderstorm", "https://www.bmkg.go.id/asset/img/icon-cuaca/hujan%20petir-{}.png"),
    "100": ("Clear Skies", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah-{}.png"),
    "101": ("Partly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah%20berawan-{}.png"),
    "102": ("Partly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/cerah%20berawan-{}.png"),
    "103": ("Mostly Cloudy", "https://www.bmkg.go.id/asset/img/icon-cuaca/berawan-{}.png"),
    "104": ("Overcast", "https://www.bmkg.go.id/asset/img/icon-cuaca/berawan%20tebal-{}.png")
}

PROVINCES = {
    "aceh": "DigitalForecast-Aceh.xml", 
    "bali": "DigitalForecast-Bali.xml", 
    "bangka_belitung": "DigitalForecast-BangkaBelitung.xml", 
    "banten": "DigitalForecast-Banten.xml", 
    "bengkulu": "DigitalForecast-Bengkulu.xml", 
    "dki_yogyakarta": "DigitalForecast-DIYogyakarta.xml", 
    "dki_jakarta": "DigitalForecast-DKIJakarta.xml", 
    "gorontalo": "DigitalForecast-Gorontalo.xml", 
    "jambi": "DigitalForecast-Jambi.xml", 
    "jawa_barat": "DigitalForecast-JawaBarat.xml", 
    "jawa_tengah": "DigitalForecast-JawaTengah.xml", 
    "jawa_Timur": "DigitalForecast-JawaTimur.xml", 
    "kalimantan_barat": "DigitalForecast-KalimantanBarat.xml", 
    "kalimantan_selatan": "DigitalForecast-KalimantanSelatan.xml", 
    "kalimantan_tengah": "DigitalForecast-KalimantanTengah.xml", 
    "kalimantan_timur": "DigitalForecast-KalimantanTimur.xml", 
    "kalimantan_utara": "DigitalForecast-KalimantanUtara.xml", 
    "kepulauan_riau": "DigitalForecast-KepulauanRiau.xml", 
    "lampung": "DigitalForecast-Lampung.xml", 
    "maluku": "DigitalForecast-Maluku.xml", 
    "maluku_utara": "DigitalForecast-MalukuUtara.xml", 
    "nusa_tenggara_barat": "DigitalForecast-NusaTenggaraBarat.xml", 
    "nusa_tenggara_timur": "DigitalForecast-NusaTenggaraTimur.xml", 
    "papua": "DigitalForecast-Papua.xml", 
    "papua_barat": "DigitalForecast-PapuaBarat.xml", 
    "riau": "DigitalForecast-Riau.xml", 
    "sulawesi_barat": "DigitalForecast-SulawesiBarat.xml", 
    "sulawesi_selatan": "DigitalForecast-SulawesiSelatan.xml", 
    "sulawesi_sengah": "DigitalForecast-SulawesiTengah.xml", 
    "sulawesi_tenggara": "DigitalForecast-SulawesiTenggara.xml", 
    "sulawesi_utara": "DigitalForecast-SulawesiUtara.xml", 
    "sumatera_barat": "DigitalForecast-SumateraBarat.xml", 
    "sumatera_selatan": "DigitalForecast-SumateraSelatan.xml", 
    "sumatera_utara": "DigitalForecast-SumateraUtara.xml", 
    "indonesia": "DigitalForecast-Indonesia.xml"
}


import aiohttp
from xmltodict import parse
from .weather import Weather
from .constants import PROVINCES
from .earthquake import Earthquake, EarthquakeFelt, TsunamiEarthquake
from datetime import datetime

class BMKG:
    def __repr__(self):
        return f"<BMKG english={self.english} metric={self.metric}>"

    def __init__(self, english: bool = False, metric: bool = True, session = None, session_settings: dict = {}) -> None:
        """
        BMKG wrapper for Python.
        """
        
        self.english = bool(english)
        self.metric = bool(metric)
        self.session = session or aiohttp.ClientSession(**session_settings)
        self.base_url = "https://data.bmkg.go.id/datamkg/MEWS/DigitalForecast/"
        self._day = self._current_day()
        self._image_cache = {}
    
    def _current_day(self):
        current = datetime.now()
        return f"{current.year}{current.month}{current.day}"
    
    async def get_forecast(self, location: str = None):
        if not location:
            return await self._handle_request(PROVINCES["indonesia"])
        
        location = str(location).lower().replace(" ", "_").replace("di", "dki").replace("jogja", "yogya").lstrip("provinsi ")
        if location in PROVINCES.keys():
            return await self._handle_request(PROVINCES[location])
        
        for province in PROVINCES.keys():
            if location in province:
                return await self._handle_request(PROVINCES[province])
        return await self._handle_request(PROVINCES["indonesia"])

    async def get_climate_info(self):
        if self._image_cache.get("climate_info") and (self._current_day() == self._day):
            return self._image_cache["climate_info"]
        
        response = await self.session.get("https://cdn.bmkg.go.id/DataMKG/CEWS/pch/pch.bulan.1.cond1.png")
        byte = await response.read()
        self._image_cache["climate_info"] = byte
        return byte
    
    async def get_satellite_image(self):
        if self._image_cache.get("satellite") and (self._current_day() == self._day):
            return self._image_cache["satellite"]
        
        response = await self.session.get("https://inderaja.bmkg.go.id/IMAGE/HIMA/H08_EH_Indonesia.png")
        byte = await response.read()
        self._image_cache["satellite"] = byte
        return byte
    
    async def get_wave_height_forecast(self):
        if self._image_cache.get("wave_height") and (self._current_day() == self._day):
            return self._image_cache["wave_height"]
        
        response = await self.session.get("https://cdn.bmkg.go.id/DataMKG/MEWS/maritim/gelombang_maritim.png")
        byte = await response.read()
        self._image_cache["wave_height"] = byte
        return byte

    async def get_wind_forecast(self):
        if self._image_cache.get("wind_forecast") and (self._current_day() == self._day):
            return self._image_cache["wind_forecast"]
        
        response = await self.session.get("https://cdn.bmkg.go.id/DataMKG/MEWS/angin/streamline_d1.jpg")
        byte = await response.read()
        self._image_cache["wind_forecast"] = byte
        return byte
    
    async def get_forest_fires(self):
        if self._image_cache.get("forest_fires") and (self._current_day() == self._day):
            return self._image_cache["forest_fires"]
        
        response = await self.session.get("https://cdn.bmkg.go.id/DataMKG/MEWS/spartan/36_indonesia_ffmc_01.png")
        byte = await response.read()
        self._image_cache["forest_fires"] = byte
        return byte
    
    async def get_recent_earthquake_map(self):
        if self._image_cache.get("latest_earthquake") and (self._current_day() == self._day):
            return self._image_cache["latest_earthquake"]
        
        response = await self.session.get("https://data.bmkg.go.id/eqmap.gif")
        byte = await response.read()
        self._image_cache["latest_earthquake"] = byte
        return byte

    async def get_recent_earthquake(self):
        response = await self.session.get("https://data.bmkg.go.id/gempaterkini.xml")
        text = await response.text()
        return Earthquake(text, metric=self.metric)
    
    async def get_recent_tsunami(self):
        response = await self.session.get("https://data.bmkg.go.id/lasttsunami.xml")
        text = await response.text()
        return TsunamiEarthquake(text, metric=self.metric)
    
    async def get_earthquakes_felt(self):
        response = await self.session.get("https://data.bmkg.go.id/gempadirasakan.xml")
        text = await response.text()
        result = parse(text)["Infogempa"]
        earthquakes = []
        
        for earthquake in result["Gempa"]:
            earthquakes.append(EarthquakeFelt(earthquake, metric=self.metric))
        return earthquakes

    async def get_recent_earthquakes(self):
        response = await self.session.get("https://data.bmkg.go.id/gempaterkini.xml")
        text = await response.text()
        result = parse(text)["Infogempa"]
        earthquakes = []
        
        for earthquake in result["gempa"]:
            earthquakes.append(Earthquake(earthquake, as_list_element=True, metric=self.metric))
        return earthquakes

    async def _handle_request(self, xml_path: str):
        response = await self.session.get(self.base_url + xml_path)
        text = await response.text()
        return Weather(text, english=self.english, metric=self.metric)
    
    async def close(self):
        await self.session.close()
        del (
            self._day,
            self.base_url,
            self.session,
            self.english,
            self.metric,
            self._image_cache
        )


from setuptools import setup
setup(
  name = 'bmkg',
  packages = ['bmkg'],
  version = '0.0.2',
  license='MIT',
  description = 'Unofficial BMKG API Python Wrapper.',
  long_description = open('README.md', 'r', encoding='utf-8').read(),
  long_description_content_type='text/markdown',
  author = 'vierofernando',
  author_email = 'vierofernando9@gmail.com',
  url = 'https://github.com/vierofernando/bmkg',
  download_url = 'https://github.com/vierofernando/bmkg/archive/0.0.2.tar.gz',
  keywords = ['Weather', 'BMKG', 'Indonesia', 'API', 'API Wrapper', 'Wrapper'],
  install_requires=[
    'aiohttp',
    'xmltodict'
  ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9'
  ],
  python_requires='>=3.7',
)


from datetime import datetime, timedelta, timezone
from xmltodict import parse
from .constants import TIMEZONE_OFFSETS, DIRECTION

class TsunamiEarthquake:
    def __init__(self, data, **settings):
        data = parse(data)["Infotsunami"]["Gempa"]
        date = "-".join(data["Tanggal"].split("-")[:-1]) + ("20" + data["Tanggal"].split("-")[2])
        self.date = datetime.strptime(date + data["Jam"].split()[0], "%d-%b%Y%H:%M:%S") - timedelta(hours=TIMEZONE_OFFSETS[data["Jam"].split()[1]])
        self.timezone = timezone(timedelta(hours=TIMEZONE_OFFSETS[data["Jam"].split()[1]]))
        
        self.depth = float(data["Kedalaman"].split()[0]) // (1 if settings["metric"] else 1.609)
        self.magnitude = float(data["Magnitude"].split()[0])
        self.location = {
            "length": float(data["Area"].split()[0]) // (1 if settings["metric"] else 1.609),
            "direction": DIRECTION[data["Area"].split()[2]],
            "location": data["Area"].split()[-1]
        }
        self.url = data["Linkdetail"]
    
    def __repr__(self):
        return f"<TsunamiEarthquake magnitude={self.magnitude} depth={self.depth} date={repr(self.date)}>"

class EarthquakeFelt:
    def __init__(self, data, **settings):
        self.latitude    = float(data["point"]["coordinates"].split(", ")[0])
        self.longitude   = float(data["point"]["coordinates"].split(", ")[1])
        self.magnitude   = float(data["Magnitude"])
        self.depth       = float(data["Kedalaman"].split()[0]) // (1 if settings["metric"] else 1.609)
        self.description = data.get("Keterangan")
        self.felt_at     = data["Dirasakan"].lstrip(" ").rstrip(",").split(", ")
        self.date        = datetime.strptime(data["Tanggal"].split()[0], "%d/%m/%Y-%H:%M:%S") - timedelta(hours=TIMEZONE_OFFSETS[data["Tanggal"].split()[1]])
        self.timezone    = timezone(timedelta(hours=TIMEZONE_OFFSETS[data["Tanggal"].split()[1]]))
    
    def __repr__(self):
        return f"<EarthquakeFelt latitude={self.latitude} longitude={self.longitude} depth={self.depth} description={self.description}>"

class Earthquake:
    def __init__(self, data, as_list_element: bool = False, **settings):
        data = data if as_list_element else parse(data)["Infogempa"]["gempa"][0]
        self.latitude = float(data["point"]["coordinates"].split(",")[0])
        self.longitude = float(data["point"]["coordinates"].split(",")[1])
        self.magnitude = float(data["Magnitude"].split()[0])
        self.depth = float(data["Kedalaman"].split()[0]) // (1 if settings["metric"] else 1.609)
        self.tsunami = (not data["Potensi"].startswith("tidak")) if data.get("Potensi") else None
        
        date = "-".join(data["Tanggal"].split("-")[:-1]) + ("20" + data["Tanggal"].split("-")[2])
        self.date = datetime.strptime(date + data["Jam"].split()[0], "%d-%b%Y%H:%M:%S") - timedelta(hours=TIMEZONE_OFFSETS[data["Jam"].split()[1]])
        self.timezone = timezone(timedelta(hours=TIMEZONE_OFFSETS[data["Jam"].split()[1]]))
        self.locations = [{
            "length": int(data[i].split()[0]) // (1 if settings["metric"] else 1.609),
            "direction": DIRECTION[data[i].split()[2]],
            "location": data[i].split()[-1]
        } for i in filter(lambda x: x.startswith("Wilayah"), data.keys())]
    
    def __repr__(self):
        return f"<Earthquake magnitude={self.magnitude} depth={self.depth} tsunami={self.tsunami} locations=[{len(self.locations)}]>"